#!/usr/bin/env python
"""Program entry point"""

from __future__ import print_function

import argparse
from cmd import Cmd
import logging
import pipes
from pydoc import getdoc
import sys
from fabric.state import env
from fabric.contrib.console import confirm

from ficloud import metadata
from ficloud.deployment import FicloudDeployment, project_name_by_dir

from ficloud.fig_ext import FigCommand
from fig.cli.docopt_command import NoSuchCommand
from fig.cli.errors import UserError
from fig.cli.main import TopLevelCommand, parse_doc_section
from fig.packages.docker import APIError
from fig.project import NoSuchService, DependencyError
import os
import re
import shlex

log = logging.getLogger(__name__)

def format_epilog():
    """Program entry point.

    :param argv: command-line arguments
    :type argv: :class:`list`
    """
    author_strings = []
    for name, email in zip(metadata.authors, metadata.emails):
        author_strings.append('Author: {0} <{1}>'.format(name, email))
    epilog = '''
{project} {version}

{authors}
URL: <{url}>
'''.format(
        project=metadata.project,
        version=metadata.version,
        authors='\n'.join(author_strings),
        url=metadata.url)
    return epilog


def fig_main(env, fig_cmd, app_name, **kwargs):
    # Disable requests logging

    try:
        command = FigCommand(env, app_name)
        command.dispatch(fig_cmd.split(' '), None)
    except KeyboardInterrupt:
        log.error("\nAborting.")
        exit(1)
    except (UserError, NoSuchService, DependencyError) as e:
        log.error(e.msg)
        exit(1)
    except NoSuchCommand as e:
        log.error("No such command: %s", e.command)
        log.error("")
        log.error("\n".join(parse_doc_section("commands:", getdoc(e.supercommand))))
        exit(1)
    except APIError as e:
        log.error(e.explanation)
        exit(1)


def populate_client_parser(subparsers):


    # # # ficloud use ubuntu@myserver.com
    # fig_cmd = subparsers.add_parser('fig', help='Executes fig commands')
    # fig_cmd.add_argument('--env', help='Environment name', default='dev')
    # fig_cmd.add_argument('--app-name', help='App name')
    # fig_cmd.add_argument('fig_cmd', help='Fig command to execeute')
    # fig_cmd.set_defaults(func=func=fig_main)

    cmd = subparsers.add_parser('start', help='Run services as daemons')
    cmd.add_argument('services', help='Service names', nargs='*')
    cmd.add_argument('--no-logs', dest='logs', action='store_false', default=True, help='No logs')
    cmd.add_argument('--rebuild', action='store_true', default=False, help='Rebuild container')
    cmd.set_defaults(func='start')

    cmd = subparsers.add_parser('run', help='Run command on a service')
    cmd.add_argument('service', help='Service name')
    cmd.add_argument('command', help='Command to run')
    cmd.add_argument('--no-tty', dest='disable_tty', action='store_false', default=True, help='No tty')
    cmd.set_defaults(func='run')

    cmd = subparsers.add_parser('stop', help='Stop services')
    cmd.add_argument('services', help='Service names', nargs='*')
    cmd.set_defaults(func='stop')

    cmd = subparsers.add_parser('destroy', help='Destory services and containers')
    cmd.add_argument('services', help='Service names', nargs='*')
    cmd.set_defaults(func='destroy')

    cmd = subparsers.add_parser('rebuild', help='Destory services and containers')
    cmd.add_argument('services', help='Service names', nargs='*')
    cmd.set_defaults(func='rebuild')

    cmd = subparsers.add_parser('logs', help='Fetch logs from containers')
    cmd.add_argument('services', help='Service names', nargs='*')
    cmd.set_defaults(func='logs')


    # 'PS1=(.env)\[\e]0;\u@\h: \w\a\]${debian_chroot:+($debian_chroot)}\u@\h:\w\$'

    def exec_bash(**kwargs):
        shell = os.environ.get('SHELL')
        print('*' * 75)
        print('Executing %s ... Hit Ctrl+d when you are done, to return ficloud.' % shell)
        print('*' * 75 + '\n')
        os.system(shell)

    cmd = subparsers.add_parser('sh', help='Fetch logs from containers')
    cmd.set_defaults(func=exec_bash)

    cmd = subparsers.add_parser('volumes', help='Show volumes of current project')
    cmd.add_argument('services', help='Service names', nargs='*')
    cmd.add_argument('--json', action='store_true', default=False)
    cmd.set_defaults(func='list_volumes')

    cmd = subparsers.add_parser('volume-push', help='Push volume to remote server')
    cmd.add_argument('volumes', help='Volume specs', nargs='*')
    cmd.set_defaults(func='push_volumes')

    cmd = subparsers.add_parser('volume-pull', help='Push volume to remote server')
    cmd.add_argument('volumes', help='Volume specs', nargs='*')
    cmd.set_defaults(func='pull_volumes')

    cmd = subparsers.add_parser('status', help='Show current status of services')
    cmd.set_defaults(func='status')


def main(argv):

    console_handler = logging.StreamHandler(stream=sys.stderr)
    console_handler.setFormatter(logging.Formatter())
    console_handler.setLevel(logging.INFO)

    root_logger = logging.getLogger()
    root_logger.addHandler(console_handler)
    root_logger.setLevel(logging.DEBUG)

    logging.getLogger("requests").propagate = False

    arg_parser = argparse.ArgumentParser(
        prog=argv[0],

        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=metadata.description,
        epilog=format_epilog())

    arg_parser.add_argument('-e', '--env', help='Environment to use', default='dev')

    arg_parser.add_argument(
        '-V', '--version',
        action='version',
        version='{0} {1}'.format(metadata.project, metadata.version))

    subparsers = arg_parser.add_subparsers()



    populate_client_parser(subparsers)

    # ficloud use ubuntu@myserver.com
    use_cmd = subparsers.add_parser('use', help='Sets target hostname')
    use_cmd.add_argument('host', help='Hostname with username ex. user@some.server')
    use_cmd.set_defaults(func='use_host')

    # ficloud app create myapp
    app_create_cmd = subparsers.add_parser('@', help='Executes remote command')
    app_create_cmd.add_argument('command', help='Name of application', nargs='*')
    app_create_cmd.set_defaults(func='remote')

    class foo(Cmd):

        intro = '''
   __ _      _                 _
  / _(_) ___| | ___  _   _  __| |
 | |_| |/ __| |/ _ \| | | |/ _` |
 |  _| | (__| | (_) | |_| | (_| |
 |_| |_|\___|_|\___/ \__,_|\__,_|

 Cloud that loves your data.

'''

        def update_prompt(self):
            if os.path.exists('ficloud.yml'):
                project = project_name_by_dir()
            else:
                project = '~'

            self.prompt = '(%s:%s) ' % (project, env.host_string or '~')


        def __init__(self, completekey='tab', stdin=None, stdout=None):

            self.client = FicloudDeployment()
            self.client.init(os.getcwd(), 'dev')

            self.update_prompt()
            Cmd.__init__(self, completekey, stdin, stdout)

        def completedefault(self, *ignored):
            return Cmd.completedefault(self, *ignored)


        def exec_argparse(self, args):
            args = arg_parser.parse_args(args=args)
            args.argv0 = argv[0]
            if isinstance(args.func, str):
                getattr(self.client, args.func)(**vars(args))
            else:
                args.func(**vars(args))

        def onecmd(self, line):
            try:
                # reset project config
                self.client._project = None

                if line.strip() != '':
                    args = shlex.split(line.strip())
                    if args[0] == 'EOF':
                        if confirm('\nCtrl+d. Exit?', True):
                            return True
                    if args[0] in ('docker', 'ls', 'll', 'l', 'git'):
                        os.system(' '.join(args))
                    else:
                        self.exec_argparse(args)
                else:
                    arg_parser.print_help()
            except KeyboardInterrupt as e:
                print('Ctrl+c. Task interrupted.')
            except SystemExit as e:
                pass
            except Exception as e:
                print('Error: %s' % str(e))

            self.update_prompt()

    if len(argv) > 1:
        foo().exec_argparse(argv[1:])
    else:
        if not os.path.exists('ficloud.yml'):
            if not confirm('Directory do not contain ficloud.yml, ficloud may be not so useful, continue to ficloud?', False):
                raise SystemExit(0)
        foo().cmdloop()





def entry_point():
    """Zero-argument entry point for use with setuptools/distribute."""
    raise SystemExit(main(sys.argv))


if __name__ == '__main__':
    entry_point()



from ficloud.fig_ext import transform_config


def test_transform_config():
    config = {'redis': {'image': 'orchardup/redis'},
              'web': {'build': '.',
                      'links': ['redis'],
                      'volumes': ['.:/code'],
                      '~dev': {'ports': ['5000:6000']},
                      '~prod': {'ports': ['5000:5000']}}}

    assert {'redis': {'image': 'orchardup/redis'},
            'web': {'build': '.',
                    'links': ['redis'],
                    'volumes': ['.:/code'],
                    'ports': ['5000:6000']}} == transform_config(config, env='dev')

    assert {'redis': {'image': 'orchardup/redis'},
            'web': {'build': '.',
                    'links': ['redis'],
                    'volumes': ['.:/code'],
                    'ports': ['5000:5000']}} == transform_config(config, env='prod')


def test_transform_config_root_level():
    config = {
        'foo': [123],
        '~1': {
            'hoho': 'hoho'
        },
        '~2': {
            'baz': 'baz'
        }
    }

    assert {
        'foo': [123],
        'hoho': 'hoho'
    } == transform_config(config, env='1')

    assert {
        'foo': [123],
        'baz': 'baz'
    } == transform_config(config, env='2')


def test_transform_config_default():
    config = {
        'foo': [123],
        '~dev': {
            'hoho': 'hoho'
        },
        '~2': {
            'baz': 'baz'
        }
    }

    assert {
        'foo': [123]
    } == transform_config(config)



import errno
import logging
import pprint
import yaml
from fig.cli.main import TopLevelCommand
from fig.cli.utils import cached_property
from fig.project import Project
import os

log = logging.getLogger(__name__)


def transform_config(config, env=None):

    if isinstance(config, dict):
        obj = {}
        for key, val in config.items():
            if isinstance(key, str):
                if key[0] == '~':
                    if key[1:] == env:
                        obj.update(val)
                else:
                    obj[key] = transform_config(val, env)
        return obj
    elif isinstance(config, list):
        return map(lambda item: transform_config(item, env), config)
    else:
        return config


class FigCommand(TopLevelCommand):

    def __init__(self, env_name, project_name=None):
        # copy help from parent class
        self.__doc__ = TopLevelCommand.__doc__

        self.env_name = env_name
        self.custom_project_name = project_name

        super(FigCommand, self).__init__()

    @cached_property
    def project(self):
        try:
            yaml_path = self.check_yaml_filename()
            config = yaml.load(open(yaml_path))

            config = transform_config(config, env=self.env_name)

            return Project.from_config(self.custom_project_name or self.project_name, config, self.client)

        except IOError as e:
            if e.errno == errno.ENOENT:
                log.error("Can't find %s. Are you in the right directory?", os.path.basename(e.filename))
            else:
                log.error(e)

            exit(1)



"""Project metadata

Information describing the project.
"""

# The package name, which is also the "UNIX name" for the project.
package = 'ficloud'
project = "ficloud"
project_no_spaces = project.replace(' ', '')
version = '0.1.13'
description = 'Production cloud deployments of fig infrastructure with docker'
authors = ['Alex Rudakov']
authors_string = ', '.join(authors)
emails = ['ribozz@gmail.com']
license = 'Apache License'
copyright = '2013 ' + authors_string
url = 'http://pywizard.com/ficloud/'




def format_service_status(service):
    if len(service.containers()):
        s = []
        for ct in service.containers(stopped=True):
            if ct.is_running:
                s.append('UP[%s]' % ct.human_readable_ports)
            else:
                s.append('DOWN')

        service_status = ','.join(s)

    else:
        service_status = 'NO CONTAINERS'
    return service_status




import json
import os
import sys
import re

from fig.cli.command import Command
from fig.packages.docker import Client
from fig.project import Project
from prettytable import PrettyTable

from jinja2 import Environment as JinjaEnv, StrictUndefined, PackageLoader, FileSystemLoader
import yaml
from ficloud.deployment import FicloudDeployment
from ficloud.util import format_service_status


class FicloudHost():

    def __init__(self):
        self.client = Client()
        self.balancer_root = ''

    def _get_git_dir(self):
        return os.path.expanduser('~/apps')

    def _get_app_git_dir(self, name):
        return '%s/%s' % (self._get_git_dir(), name)

    def _get_deployment_dir(self):
        return os.path.expanduser('~/deploy')

    def get_app_deployment_dir(self, name, version):
        return '%s/%s/%s' % (self._get_deployment_dir(), name, version)

    def _get_apps_conf_dir(self):
        return os.path.expanduser('~/apps-conf')

    def format_domain_config_path(self, domain_name):
        conf_dir = self._get_apps_conf_dir()
        if not os.path.exists(conf_dir):
            os.mkdir(conf_dir)
        return '%s/%s.conf' % (conf_dir, domain_name)

    def create_app(self, name, argv0, **kwargs):
        """
        

        """
        repo_dir = self._get_app_git_dir(name)

        if not os.path.exists(repo_dir):
            os.system('git init --bare %s' % repo_dir)
            with open('%s/hooks/post-receive' % repo_dir, 'w') as f:
                f.write('#!/bin/bash\n'
                        '%s git-post-receive' % argv0)
            os.system('chmod +x %s/hooks/post-receive' % repo_dir)
        else:
            print('Application already created')

    def remove_app(self, name, **kwargs):
        """
        

        """
        repo_dir = self._get_app_git_dir(name)

        app_dir = self._get_deployment_dir() + '/' + name
        if os.path.exists(app_dir):
            for version in os.listdir(app_dir):
                self.undeploy_app(name, version)
            os.system('rm -rf %s' % app_dir)

        if os.path.exists(repo_dir):
            os.system('rm -rf %s' % repo_dir)


    def deploy_app(self, name, version, **kwargs):
        """
        

        """
        repo_dir = self._get_app_git_dir(name)
        target_dir = self.get_app_deployment_dir(name, version)

        if os.path.exists(target_dir):
            os.system('rm -rf %s' % target_dir)

        os.system('git clone -b %s %s %s' % (version, repo_dir, target_dir))

        deployment = self.get_deployment(name, version)

        deployment.status()


    def undeploy_app(self, name, version, **kwargs):
        """
        

        """
        target_dir = self.get_app_deployment_dir(name, version)

        if os.path.exists(target_dir):

            deployment = self.get_deployment(name, version)
            deployment.destroy()

            os.system('rm -rf %s' % target_dir)




    def list_apps(self, **kwargs):

        table = PrettyTable(["App", "Version", "State"])

        if os.path.exists(self._get_git_dir()):
            last_app = None

            for app in os.listdir(self._get_git_dir()):
                app_dir = self._get_deployment_dir() + '/' + app

                if os.path.exists(app_dir):
                    versions = os.listdir(app_dir)

                    if len(versions):
                        for version in versions:
                            try:
                                project = self.get_deployment(app, version).project

                                status = {}
                                for service in project.get_services():
                                    status[service.name] = format_service_status(service)

                                app_status = ''
                                for service, st in status.items():
                                    app_status += '%s=%s ' % (service, st)

                                table.add_row(('' if last_app == app else app, version, app_status))
                                last_app = app
                            except IOError:
                                print('Application %s : %s has no config.yml. Wrong branch deployed?' % (app, version))
                    else:
                        table.add_row((app, '-', 'NOT DEPLOYED'))
                else:
                    table.add_row((app, '-', 'NOT DEPLOYED'))

        print(table)

    def get_balancer_destinations(self, app_name, app_version, service, port):

        project = self.get_deployment(app_name, app_version).project

        service_ports = []
        for c in project.get_service(service).containers():
            if c.is_running:
                ports = c.inspect()['NetworkSettings']['Ports']

                if not '%s/tcp' % port in ports:
                    raise RuntimeError('Container has no port %s exposed')

                service_ports.append(('127.0.0.1', int(ports['%s/tcp' % port][0]['HostPort'])))

        return service_ports

    def balancer_remove(self, domain, **kwargs):
        os.unlink(self.format_domain_config_path(domain))

    def balancer_set(self, domain, path, **kwargs):

        p = re.compile(r'^[\w\-\._]+$')
        result = p.match(domain)

        if not result:
            print('\nEnter valid domain name\n')
            sys.exit(1)

        p = re.compile(r'^([\w\-]+):(\d+)@([\w\-]+)#([\w\-]+)$')
        result = p.match(path)

        if not result:
            print('\nPath should be in form: service:port@appname#app_version\n')
            sys.exit(1)

        app_name = result.group(3)
        app_version = result.group(4)
        service_name = result.group(1)
        service_port = result.group(2)

        destinations = self.get_balancer_destinations(app_name, app_version, service_name, service_port)
        if not destinations:
            print('Looks like application is not running')
            sys.exit(1)

        config = {
            'target': path,
            'domain': domain,
            'app_name': app_name,
            'app_version': app_version,
            'backends': destinations
        }

        with open(self.format_domain_config_path(domain), 'w+') as f:
            json.dump(config, f)

    def balancer_list(self, **kwargs):

        table = PrettyTable(["Domain", "Target", "Resolved backends"])
        if os.path.exists(self._get_apps_conf_dir()):
            for conf_file in [x for x in os.listdir(self._get_apps_conf_dir()) if x.endswith('.conf')]:
                with open(self._get_apps_conf_dir() + '/' + conf_file) as f:
                    conf = json.load(f)
                    table.add_row(
                        (conf['domain'], conf['target'], ', '.join(['%s:%s' % tuple(x) for x in conf['backends']])))

        print(table)

    def inotify_dump(self, source, haproxytpl, argv0, **kwargs):

        print('%(conf-dir)s IN_MODIFY,IN_CREATE,IN_DELETE,IN_NO_LOOP %(ficloud-server)s balancer-dump %(conf-dir)s' % {
            'ficloud-server': argv0,
            'conf-dir': source
        })
        print('%(haproxytpl)s IN_MODIFY,IN_CREATE,IN_DELETE,IN_NO_LOOP %(ficloud-server)s balancer-dump %(conf-dir)s' % {
            'haproxytpl': haproxytpl,
            'ficloud-server': argv0,
            'conf-dir': source
        })


    def balancer_dump(self, source, path, **kwargs):

        context = {}

        full_conf = {}

        if os.path.exists(source):
            for conf_file in [x for x in os.listdir(source) if x.endswith('.conf')]:
                with open(source + '/' + conf_file) as f:
                    conf = json.load(f)
                    full_conf[conf['domain']] = {
                        'name': re.sub('[^a-z0-9_]', '_', conf['domain']),
                        'domains': [conf['domain']],
                        'backends': [{
                                         'name': 'backend_%s' % i,
                                         'ip': x[0],
                                         'port': x[1]
                                     } for i, x in enumerate(conf['backends'])]
                    }

        context['apps'] = full_conf.values()

        engine = JinjaEnv(
            loader=FileSystemLoader('/etc/haproxy'),
            undefined=StrictUndefined
        )

        with open(path, 'w') as f:
            f.write(engine.get_template('haproxy.cfg.tpl').render(context))

        os.system('service haproxy reload')


    def git_post_receive(self, **kwargs):
        """
        Command is used inside git's post-receive hook
        """
        #!/home/alex/dev/ficloud/.env/bin/python

        p = re.compile(r'refs/heads/([^\s]+)')

        data = sys.stdin.read()
        print('>>%s<<' % data)
        m = p.search(data)
        if m:
            branch = m.group(1)
            app_name = os.path.basename(os.getcwd())
            print('\n\nDeploying app %s version %s ..\n\n' % (app_name, branch))

            self.deploy_app(app_name, branch)
        else:
            print('\n\nNB! No branch to deploy!\n\n')

    def get_deployment(self, name, version):
        deployment = FicloudDeployment()
        target_dir = self.get_app_deployment_dir(name, version)
        deployment.init(target_dir, 'prod', '%s0%s' % (name, version))

        return deployment











from pytest import raises
# The parametrize function is generated, so this doesn't work:
#
#     from pytest.mark import parametrize
#
import pytest
parametrize = pytest.mark.parametrize

from ficloud import metadata
from ficloud.client_cli import main


@parametrize('helparg', ['-h', '--help'])
def test_help(helparg, capsys):
    with raises(SystemExit) as exc_info:
        main(['progname', helparg])
    out, err = capsys.readouterr()
    # Should have printed some sort of usage message. We don't
    # need to explicitly test the content of the message.
    assert 'usage' in out
    # Should have used the program name from the argument
    # vector.
    assert 'progname' in out
    # Should exit with zero return code.
    assert exc_info.value.code == 0

@parametrize('versionarg', ['-V', '--version'])
def test_version(versionarg, capsys):
    with raises(SystemExit) as exc_info:
        main(['progname', versionarg])
    out, err = capsys.readouterr()
    # Should print out version.
    assert err == '{0} {1}\n'.format(metadata.project, metadata.version)
    # Should exit with zero return code.
    assert exc_info.value.code == 0



#!/usr/bin/env python
"""Program entry point"""

from __future__ import print_function

import argparse
import logging
import sys

from ficloud import metadata
from ficloud.client_cli import populate_client_parser
from ficloud.deployment import FicloudDeployment
from ficloud.host import FicloudHost
import os


def main(argv):

    arg_parser = argparse.ArgumentParser(
        prog=argv[0],
        formatter_class=argparse.RawDescriptionHelpFormatter,
        description=metadata.description)

    arg_parser.add_argument('--log-level', type=str, help='Log level. Useful ones: DEBUG, INFO, ERROR', default='INFO')

    arg_parser.add_argument(
        '-V', '--version',
        action='version',
        version='{0} {1}'.format(metadata.project, metadata.version))

    subparsers = arg_parser.add_subparsers()

    host = FicloudHost()

    app_list_cmd = subparsers.add_parser('app-list', help='List applications deployed')
    app_list_cmd.set_defaults(func=host.list_apps)



    def ficloud_app_wrpapper(name, version, func, **kwargs):
        """
        Creates new application. Basically, creates new git repo.

        """

        deployment = host.get_deployment(name, version)

        getattr(deployment, func)(**kwargs)

    app_cmd = subparsers.add_parser('app', help='List applications versions deployed')
    app_cmd.add_argument('name', help='Application name')
    app_cmd.add_argument('version', help='Version')
    app_cmd.set_defaults(wrapper=ficloud_app_wrpapper)
    app_cmd_subparser = app_cmd.add_subparsers()
    populate_client_parser(app_cmd_subparser)

    app_versions_cmd = subparsers.add_parser('app-create', help='Create new application')
    app_versions_cmd.add_argument('name', help='Application name')
    app_versions_cmd.set_defaults(func=host.create_app)

    app_versions_cmd = subparsers.add_parser('app-remove', help='Remove an application')
    app_versions_cmd.add_argument('name', help='Application name')
    app_versions_cmd.set_defaults(func=host.remove_app)

    app_versions_cmd = subparsers.add_parser('app-deploy', help='Deploy an application')
    app_versions_cmd.add_argument('name', help='Application name')
    app_versions_cmd.add_argument('version', help='Version name')
    app_versions_cmd.set_defaults(func=host.deploy_app)

    app_versions_cmd = subparsers.add_parser('app-undeploy', help='Deploy an application')
    app_versions_cmd.add_argument('name', help='Application name')
    app_versions_cmd.add_argument('version', help='Version name')
    app_versions_cmd.set_defaults(func=host.undeploy_app)

    app_create_cmd = subparsers.add_parser('balancer-set', help='Sets new destination for a domain')
    app_create_cmd.add_argument('domain', help='domain name')
    app_create_cmd.add_argument('path', help='destination path')
    app_create_cmd.set_defaults(func=host.balancer_set)

    app_create_cmd = subparsers.add_parser('balancer-remove', help='Remove destination for a domain')
    app_create_cmd.add_argument('domain', help='domain name')
    app_create_cmd.set_defaults(func=host.balancer_remove)

    app_create_cmd = subparsers.add_parser('balancer-list', help='List all balancers')
    app_create_cmd.set_defaults(func=host.balancer_list)

    app_create_cmd = subparsers.add_parser('balancer-dump', help='Dump haproxy config')
    app_create_cmd.add_argument('source', help='source path')
    app_create_cmd.add_argument('path', help='config path', default='/etc/haproxy/haproxy.cfg', nargs='?')
    app_create_cmd.set_defaults(func=host.balancer_dump)

    app_create_cmd = subparsers.add_parser('inotify-dump', help='Dump inotify config')
    app_create_cmd.add_argument('source', help='source path', default='/home/ficloud/apps-conf', nargs='?')
    app_create_cmd.add_argument('haproxytpl', help='haproxy temp[late path', default='/etc/haproxy/haproxy.cfg.tpl', nargs='?')
    app_create_cmd.set_defaults(func=host.inotify_dump)

    app_create_cmd = subparsers.add_parser('git-post-receive', help='Used by git')
    app_create_cmd.set_defaults(func=host.git_post_receive)


    if len(argv) > 1:
        args = arg_parser.parse_args(args=argv[1:])
        logging.getLogger().level = logging.DEBUG

        args.argv0 = argv[0]

        if hasattr(args, 'wrapper'):
            args.wrapper(**vars(args))
        else:
            args.func(**vars(args))

        return 0

    else:
        arg_parser.print_help()
        return 1


def entry_point():
    """Zero-argument entry point for use with setuptools/distribute."""
    raise SystemExit(main(sys.argv))


if __name__ == '__main__':
    entry_point()



from fig.cli.log_printer import LogPrinter
from fig.cli.socketclient import SocketClient
from fig.packages.docker import Client
import getpass
import json
from prettytable import PrettyTable
from fabric.api import run, env
import fabric
from fig.project import Project
from fabric.contrib.console import confirm
import os
import yaml
from os.path import expanduser
import cuisine as remote
import re
from ficloud.fig_ext import transform_config
from ficloud.util import format_service_status


def project_name_by_dir():
    project_name = os.path.basename(os.getcwd())
    project_name = re.sub(r'[^a-zA-Z0-9]', '', project_name)
    return project_name


class FicloudDeployment():
    def __init__(self, config_file='~/.ficloud.yml'):
        self.ficloud_yml = expanduser(config_file)

        # load config
        if os.path.exists(self.ficloud_yml):
            with open(self.ficloud_yml) as f:
                self.config = yaml.load(f)
                if self.config is None:
                    self.config = {}
        else:
            self.config = {}

        if 'host' in self.config:
            env.host_string = self.config['host']

        env.output_prefix = False
        fabric.state.output.running = False

        self.client = Client()

        self.project_dir = None
        self.project_name = None
        self.env_name = None
        self._project = None

    def init(self, project_dir, env_name, project_name=None):
        self.project_dir = project_dir
        self.env_name = env_name

        if not project_name:
            project_name = project_name_by_dir()

        self.project_name = project_name


    def _get_fig_project(self):

        if not self.env_name:
            raise ValueError('env is not specified!')

        if not self.project_dir:
            raise ValueError('project_dir is not specified!')

        if not self.project_name:
            raise ValueError('project_name is not specified!')

        os.chdir(self.project_dir)
        with open('ficloud.yml') as f:
            config = yaml.load(f)

        config = transform_config(config['services'], env=self.env_name)
        project = Project.from_config(self.project_name, config, self.client)
        return project

    @property
    def host(self):
        if not 'host' in self.config:
            raise ValueError('host is not selected!')
        return self.config['host']

    @property
    def project(self):
        if not self._project:
            self._project = self._get_fig_project()
        return self._project

    def use_host(self, host, **kwargs):
        """
        Sets currently used host
        """
        self.config['host'] = host
        env.host_string = host

        self.save_config()

    def pull_volumes(self, volumes, **kwargs):
        return self.push_volumes(volumes, reverse=True, **kwargs)

    def push_volumes(self, volumes, reverse=False, **kwargs):
        for volume in volumes:

            p = re.compile(r'^([\w\-]+):([\w\d/_-]+)@([\w\-]+)#([\w\-]+)$')
            result = p.match(volume)

            if not result:
                print('\nVolume should be of format service:/volume/path@app#version')
                return

            (service, volume_name, app, version) = result.groups()

            data = run('ficloud-server app %s %s volumes --json' % (app, version))
            data = json.loads(data)

            volume_key = '%s@%s' % (service, volume_name)

            if not volume_key in data:
                ValueError('Remote volume %s not found on remote server' % volume_name)

            local_data = self.get_volumes()
            if not volume_key in local_data:
                ValueError('Remote volume %s not found on local machine' % volume_name)

            # reverse = pull operation
            if reverse:
                (data, local_data) = (local_data, data)

            cmd = 'sudo rsync -e "ssh -i /home/%(user)s/.ssh/id_rsa" --delete -vv -r -p --numeric-ids -e ssh' \
                  ' --rsync-path="sudo rsync" %(local_path)s/ %(host_string)s:%(remote_path)s' % {
                      'user': getpass.getuser(),
                      'host_string': env.host_string,
                      'local_path': local_data[volume_key],
                      'remote_path': data[volume_key]
                  }

            os.system(cmd)


    def status(self, **kwargs):

        table = PrettyTable(["Service", "Status"])

        for service in self.project.services:
            table.add_row((service.name, format_service_status(service)))

        print(table)

    def run(self, service, command, disable_tty=False, **kwargs):

        service = self.project.get_service(service)

        container = service.create_container(one_off=True, **{
            'command': command,
            'tty': not disable_tty,
            'stdin_open': True,
        })

        with self._attach_to_container(container.id, raw=True) as c:
            service.start_container(container, ports=None)
            c.run()

    def _attach_to_container(self, container_id, raw=False):
        socket_in = self.client.attach_socket(container_id, params={'stdin': 1, 'stream': 1})
        socket_out = self.client.attach_socket(container_id, params={'stdout': 1, 'logs': 1, 'stream': 1})
        socket_err = self.client.attach_socket(container_id, params={'stderr': 1, 'logs': 1, 'stream': 1})

        return SocketClient(
            socket_in=socket_in,
            socket_out=socket_out,
            socket_err=socket_err,
            raw=raw,
        )

    def start(self, services=None, logs=True, rebuild=False, **kwargs):

        project = self.project

        if rebuild:
            project.stop(services)
            project.remove_stopped(services)
            project.build(services)

        all_containers = []
        for service in project.get_services(services):
            if len(service.containers(stopped=True)) == 0:
                service.create_container()

            all_containers += service.containers(stopped=True)

        log_printer = None
        if logs:
            log_printer = LogPrinter(all_containers)

        project.start(services)

        if log_printer:
            try:
                print("Following logs from containers.")
                log_printer.run()
            except KeyboardInterrupt:
                print("Interrupted by Ctrl + C. Containers are still running. Use `ficloud stop` to stop them.")


    def stop(self, services=None, **kwargs):
        project = self.project
        project.stop(services)

    def destroy(self, services=None, **kwargs):
        project = self.project
        project.stop(services)
        project.kill(services)
        project.remove_stopped(services)

    def rebuild(self, services=None, **kwargs):
        self.start(services, rebuild=True)

    def logs(self, services=None, **kwargs):
        project = self.project

        containers = project.containers(service_names=services, stopped=True)
        LogPrinter(containers, attach_params={'logs': True}).run()

    def get_volumes(self, **kwargs):
        volumes = {}
        for service in self.project.get_services():

            for c in service.containers(stopped=True):
                for volume, dir in c.inspect()['Volumes'].items():
                    volumes['%s@%s' % (service.name, volume)] = dir
        return volumes

    def list_volumes(self, **kwargs):

        if kwargs['json']:
            print(json.dumps(self.get_volumes()))
            return

        table = PrettyTable(["Volume", "Mount directory"])
        for volume, dir in self.get_volumes().items():
            table.add_row((
                volume,
                dir
            ))

        print(table)


    def remote(self, command, **kwargs):
        run('ficloud-server %s' % ' '.join(command))


    def save_config(self):
        with open(self.ficloud_yml, 'w+') as f:
            yaml.dump(self.config, f)



from __future__ import print_function

import os
import sys
import imp

from setuptools import setup, find_packages


CODE_DIRECTORY = 'ficloud'
DOCS_DIRECTORY = 'docs'
TESTS_DIRECTORY = 'tests'

# Import metadata. Normally this would just be:
#
#     from ficloud import metadata
#
# However, when we do this, we also import `ficloud/__init__.py'. If this
# imports names from some other modules and these modules have third-party
# dependencies that need installing (which happens after this file is run), the
# script will crash. What we do instead is to load the metadata module by path
# instead, effectively side-stepping the dependency problem. Please make sure
# metadata has no dependencies, otherwise they will need to be added to
# the setup_requires keyword.
metadata = imp.load_source(
    'metadata', os.path.join(CODE_DIRECTORY, 'metadata.py'))

# define install_requires for specific Python versions
python_version_specific_requires = []

# as of Python >= 2.7 and >= 3.2, the argparse module is maintained within
# the Python standard library, otherwise we install it as a separate package
if sys.version_info < (2, 7):
    python_version_specific_requires.append('argparse')


# See here for more options:
# <http://pythonhosted.org/setuptools/setuptools.html>
setup(
    name=metadata.package,
    version=metadata.version,
    author=metadata.authors[0],
    author_email=metadata.emails[0],
    maintainer=metadata.authors[0],
    maintainer_email=metadata.emails[0],
    url=metadata.url,
    description=metadata.description,
    long_description=open('README.rst').read(),
    # Find a list of classifiers here:
    # <http://pypi.python.org/pypi?%3Aaction=list_classifiers>
    classifiers=[
        'Development Status :: 1 - Planning',
        'Environment :: Console',
        'Intended Audience :: Developers',
	    'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Documentation',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Software Distribution',
    ],
    packages=find_packages(exclude=(TESTS_DIRECTORY,)),
    install_requires=[
       'fig',
       'fabric',
       'cuisine',
       'jinja2',
       'prettytable'
    ] + python_version_specific_requires,
    # Allow tests to be run with `python setup.py test'.
    tests_require=[
        'pytest==2.5.1',
        'mock==1.0.1',
        'flake8==2.1.0',
    ],
    zip_safe=False,  # don't use eggs
    entry_points={
        'console_scripts': [
            'ficloud = ficloud.client_cli:entry_point',
            'ficloud-server = ficloud.server_cli:entry_point',
        ],
        # if you have a gui, use this
        # 'gui_scripts': [
        #     'ficloud_gui = ficloud.gui:entry_point'
        # ]
    }
)